@@ -22,6 +22,15 @@
  *
  */
 
+#include "config.h"
+
+/* number of supported soundcards */
+#ifdef CONFIG_SND_DYNAMIC_MINORS
+#define SNDRV_CARDS 32
+#else
+#define SNDRV_CARDS 8		/* don't change - minor numbers */
+#endif
+
 #include <linux/version.h>
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 #include <linux/autoconf.h>
@@ -84,6 +93,11 @@ typedef __u32 __be32;
 #endif
 #endif /* < 2.6.9 */
 
+/* other missing types */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+typedef unsigned int fmode_t;
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 /* for compat layer */
 #include <linux/pci.h>
@@ -264,6 +278,10 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 typedef unsigned __nocast gfp_t;
 #endif
 
+#ifndef CONFIG_HAVE_GFP_DMA32
+#define GFP_DMA32 0		/* driver must check for 32-bit address */
+#endif
+
 #include <linux/wait.h>
 #ifndef wait_event_timeout
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
@@ -948,6 +966,7 @@ void *snd_hidden_kmalloc(size_t size, gfp_t flags);
 void *snd_hidden_kzalloc(size_t size, gfp_t flags);
 void *snd_hidden_kcalloc(size_t n, size_t size, gfp_t flags);
 char *snd_hidden_kstrdup(const char *s, gfp_t flags);
+char *snd_hidden_kstrndup(const char *s, size_t len, gfp_t flags);
 void snd_hidden_kfree(const void *obj);
 
 static inline void *snd_wrapper_kmalloc(size_t size, gfp_t flags)
@@ -963,6 +982,7 @@ static inline void snd_wrapper_kfree(const void *obj)
 #define kzalloc(size, flags) snd_hidden_kzalloc(size, flags)
 #define kcalloc(n, size, flags) snd_hidden_kcalloc(n, size, flags)
 #define kstrdup(s, flags)  snd_hidden_kstrdup(s, flags)
+#define kstrndup(s, len, flags)  snd_hidden_kstrndup(s, len, flags)
 #define kfree(obj) snd_hidden_kfree(obj)
 
 #define kmalloc_nocheck(size, flags) snd_wrapper_kmalloc(size, flags)
@@ -983,6 +1003,11 @@ char *snd_compat_kstrdup(const char *s, gfp_t gfp_flags);
 #define kstrdup(s,f) snd_compat_kstrdup(s,f)
 #endif
 
+#ifndef CONFIG_HAVE_KSTRNDUP
+char *snd_compat_kstrndup(const char *s, size_t len, gfp_t gfp_flags);
+#define kstrndup(s,l,f) snd_compat_kstrndup(s,l,f)
+#endif
+
 #ifndef CONFIG_HAVE_KZALLOC
 void *snd_compat_kzalloc(size_t n, gfp_t gfp_flags);
 #define kzalloc(s,f) snd_compat_kzalloc(s,f)
@@ -1165,15 +1190,15 @@ static inline int snd_pci_compat_register_driver(struct pci_driver *driver)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
 #ifdef CONFIG_PCI
 #ifndef CONFIG_HAVE_NEW_PCI_SAVE_STATE
-void snd_pci_compat_save_state(struct pci_dev *pci);
-void snd_pci_compat_restore_state(struct pci_dev *pci);
-static inline void snd_pci_orig_save_state(struct pci_dev *pci, u32 *buffer)
+int snd_pci_compat_save_state(struct pci_dev *pci);
+int snd_pci_compat_restore_state(struct pci_dev *pci);
+static inline int snd_pci_orig_save_state(struct pci_dev *pci, u32 *buffer)
 {
-	pci_save_state(pci, buffer);
+	return pci_save_state(pci, buffer);
 }
-static inline void snd_pci_orig_restore_state(struct pci_dev *pci, u32 *buffer)
+static inline int snd_pci_orig_restore_state(struct pci_dev *pci, u32 *buffer)
 {
-	pci_restore_state(pci, buffer);
+	return pci_restore_state(pci, buffer);
 }
 
 #undef pci_save_state
@@ -1404,7 +1429,7 @@ extern struct pt_regs *snd_irq_regs;
 #define CONFIG_SYSFS_DEPRECATED	1
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 21)
+#ifndef CONFIG_HAVE_IS_POWER_OF_2
 static inline __attribute__((const))
 int is_power_of_2(unsigned long n)
 {
@@ -1426,4 +1451,296 @@ static inline unsigned char snd_pci_revision(struct pci_dev *pci)
 #endif
 #endif /* PCI */
 
+/* BIT* macros */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
+#include <linux/bitops.h>
+/*
+#ifndef BIT
+#define BIT(nr)			(1UL << (nr))
+#endif
+*/
+#ifndef BIT_MASK
+#define BIT_MASK(nr)		(1UL << ((nr) % BITS_PER_LONG))
+#endif
+#ifndef BIT_WORD
+#define BIT_WORD(nr)		((nr) / BITS_PER_LONG)
+#endif
+#ifndef BITS_PER_BYTE
+#define BITS_PER_BYTE		8
+#endif
+#endif
+
+#ifndef CONFIG_HAVE_FFS
+#if defined(__i386__)
+static inline unsigned long __ffs(unsigned long word)
+{
+	__asm__("bsfl %1,%0"
+		:"=r" (word)
+		:"rm" (word));
+	return word;
+}
+#else
+static inline unsigned long __ffs(unsigned long word)
+{
+	__asm__("need_asm_for_your_arch_in_adriver.h");
+	return word;
+}
+#endif
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+#ifndef uninitialized_var
+#define uninitialized_var(x) x = x
+#endif
+#endif /* <2.6.0 */
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
+typedef unsigned long uintptr_t;
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
+#include <linux/time.h>
+#define do_posix_clock_monotonic_gettime getnstimeofday
+#endif
+
+/* undefine SNDRV_CARDS again - it'll be re-defined in sound/core.h */
+#undef SNDRV_CARDS
+
+/* put_unaligned_*() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 26)
+#include <asm/unaligned.h>
+static inline void put_unaligned_le16(u16 val, void *p)
+{
+	val = cpu_to_le16(val);
+	put_unaligned(val, (u16 *)p);
+}
+
+static inline void put_unaligned_le32(u32 val, void *p)
+{
+	val = cpu_to_le32(val);
+	put_unaligned(val, (u32 *)p);
+}
+
+static inline void put_unaligned_le64(u64 val, void *p)
+{
+	val = cpu_to_le64(val);
+	put_unaligned(val, (u64 *)p);
+}
+
+static inline void put_unaligned_be16(u16 val, void *p)
+{
+	val = cpu_to_be16(val);
+	put_unaligned(val, (u16 *)p);
+}
+
+static inline void put_unaligned_be32(u32 val, void *p)
+{
+	val = cpu_to_be32(val);
+	put_unaligned(val, (u32 *)p);
+}
+
+static inline void put_unaligned_be64(u64 val, void *p)
+{
+	val = cpu_to_be64(val);
+	put_unaligned(val, (u64 *)p);
+}
+
+static inline u16 get_unaligned_le16(void *p)
+{
+	u16 val = get_unaligned((u16 *)p);
+	return cpu_to_le16(val);
+}
+
+static inline u32 get_unaligned_le32(void *p)
+{
+	u32 val = get_unaligned((u32 *)p);
+	return cpu_to_le32(val);
+}
+
+static inline u64 get_unaligned_le64(void *p)
+{
+	u64 val = get_unaligned((u64 *)p);
+	return cpu_to_le64(val);
+}
+static inline u16 get_unaligned_be16(void *p)
+{
+	u16 val = get_unaligned((u16 *)p);
+	return cpu_to_be16(val);
+}
+
+static inline u32 get_unaligned_be32(void *p)
+{
+	u32 val = get_unaligned((u32 *)p);
+	return cpu_to_be32(val);
+}
+
+static inline u64 get_unaligned_be64(void *p)
+{
+	u64 val = get_unaligned((u64 *)p);
+	return cpu_to_be64(val);
+}
+#endif
+
+/* list_first_entry */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+#include <linux/list.h>
+#ifndef list_first_entry
+#define list_first_entry(ptr, type, member) \
+	list_entry((ptr)->next, type, member)
+#endif
+#endif /* < 2.6.22 */
+
+#ifndef upper_32_bits
+#define upper_32_bits(n) ((u32)(((n) >> 16) >> 16))
+#endif
+
+#ifndef CONFIG_HAVE_PAGE_TO_PFN
+#define page_to_pfn(page)       (page_to_phys(page) >> PAGE_SHIFT)
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
+/* lazy wrapper - always returns 0 */
+static inline int __strict_strtoul(const char *cp, unsigned int base,
+				 unsigned long *valp)
+{
+	*valp = simple_strtoul(cp, NULL, base);
+	return 0;
+}
+#define strict_strtoul __strict_strtoul
+#endif /* < 2.6.25 */
+
+/* pr_xxx() macros */
+#ifndef pr_emerg
+#define pr_emerg(fmt, arg...) \
+	printk(KERN_EMERG fmt, ##arg)
+#endif
+#ifndef pr_alert
+#define pr_alert(fmt, arg...) \
+	printk(KERN_ALERT fmt, ##arg)
+#endif
+#ifndef pr_crit
+#define pr_crit(fmt, arg...) \
+	printk(KERN_CRIT fmt, ##arg)
+#endif
+#ifndef pr_err
+#define pr_err(fmt, arg...) \
+	printk(KERN_ERR fmt, ##arg)
+#endif
+#ifndef pr_warning
+#define pr_warning(fmt, arg...) \
+	printk(KERN_WARNING fmt, ##arg)
+#endif
+#ifndef pr_notice
+#define pr_notice(fmt, arg...) \
+	printk(KERN_NOTICE fmt, ##arg)
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 26) && \
+    LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+static inline const char *dev_name(struct device *dev)
+{
+	return dev->bus_id;
+}
+
+/* FIXME: return value is invalid */
+#define dev_set_name(dev, fmt, args...) \
+	snprintf((dev)->bus_id, sizeof((dev)->bus_id), fmt, ##args) 
+#endif /* < 2.6.26 */
+
+#ifndef WARN
+#define WARN(condition, arg...) ({ \
+	int __ret_warn_on = !!(condition);				\
+	if (unlikely(__ret_warn_on)) {					\
+		printk("WARNING: at %s:%d %s()\n",			\
+			__FILE__, __LINE__, __func__);			\
+		printk(arg);						\
+		dump_stack();						\
+	}								\
+	unlikely(__ret_warn_on);					\
+})
+#endif
+
+/* force to redefine WARN_ON() */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
+#undef WARN_ON
+#endif
+
+#ifndef WARN_ON
+#define WARN_ON(condition) ({						\
+	int __ret_warn_on = !!(condition);				\
+	if (unlikely(__ret_warn_on)) {					\
+		printk("WARNING: at %s:%d %s()\n",			\
+			__FILE__, __LINE__, __func__);			\
+		dump_stack();						\
+	}								\
+	unlikely(__ret_warn_on);					\
+})
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
+#define dev_printk(level, dev, format, arg...)	\
+	printk(level format, ##arg)
+#define dev_emerg(dev, format, arg...)		\
+	dev_printk(KERN_EMERG , dev , format , ## arg)
+#define dev_alert(dev, format, arg...)		\
+	dev_printk(KERN_ALERT , dev , format , ## arg)
+#define dev_crit(dev, format, arg...)		\
+	dev_printk(KERN_CRIT , dev , format , ## arg)
+#define dev_err(dev, format, arg...)		\
+	dev_printk(KERN_ERR , dev , format , ## arg)
+#define dev_warn(dev, format, arg...)		\
+	dev_printk(KERN_WARNING , dev , format , ## arg)
+#define dev_notice(dev, format, arg...)		\
+	dev_printk(KERN_NOTICE , dev , format , ## arg)
+#define dev_info(dev, format, arg...)		\
+	dev_printk(KERN_INFO , dev , format , ## arg)
+#ifdef DEBUG
+#define dev_dbg(dev, format, arg...)		\
+	dev_printk(KERN_DEBUG , dev , format , ## arg)
+#else
+#define dev_dbg(dev, format, arg...)		\
+	({ if (0) dev_printk(KERN_DEBUG, dev, format, ##arg); 0; })
+#endif
+#endif /* < 2.6.0 */
+
+/*
+ * wrapper for older SPARC codes with SBUS/EBUS specific bus
+ */
+#if defined(CONFIG_SBUS) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+struct sbus_dev;
+#define snd_dma_sbus_data(sbus)        ((struct device *)(sbus))
+#define SNDRV_DMA_TYPE_SBUS            4       /* SBUS continuous */
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+#define DUMP_PREFIX_NONE	0
+#define DUMP_PREFIX_OFFSET	1
+void snd_compat_print_hex_dump_bytes(const char *prefix_str, int prefix_type,
+				     const void *buf, size_t len);
+#define print_hex_dump_bytes(a,b,c,d) snd_compat_print_hex_dump_bytes(a,b,c,d)
+#endif
+
+/*
+ * pci_ioremap_bar() wrapper
+ */
+#ifdef CONFIG_PCI
+#ifndef CONFIG_HAVE_PCI_IOREMAP_BAR
+#include <linux/pci.h>
+static inline void *pci_ioremap_bar(struct pci_dev *pdev, int bar)
+{
+	return ioremap_nocache(pci_resource_start(pdev, bar),
+			       pci_resource_len(pdev, bar));
+}
+#endif
+#endif
+
+/* pci_name() wrapper */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 23)
+#ifdef CONFIG_PCI
+#undef pci_name
+#define pci_name(pci)	((pci)->slot_name)
+#endif
+#endif
+
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
