@@ -24,7 +24,7 @@
  */
 
 #define __NO_VERSION__
-#include <sound/driver.h>
+#include "adriver.h"
 #include <asm/io.h>
 #include <linux/delay.h>
 #include <linux/init.h>
